@@ -1,4 +1,4 @@
-/* OsmoMSC - Circuit-Switched Core Network (MSC+VLR+HLR+SMSC) implementation
+/* OsmoMSC - Circuit-Switched Core Network (MSC+VLR+SMSC) implementation
  */
 
 /* (C) 2016-2019 by sysmocom s.f.m.c. GmbH <info@sysmocom.de>
@@ -60,7 +60,7 @@
 #include <osmocom/ctrl/control_if.h>
 #include <osmocom/ctrl/control_vty.h>
 #include <osmocom/ctrl/ports.h>
-#include <osmocom/msc/smpp.h>
+#include <osmocom/smpp/smpp.h>
 #include <osmocom/sigtran/osmo_ss7.h>
 #include <osmocom/mgcp_client/mgcp_client.h>
 #include <osmocom/msc/sgs_iface.h>
@@ -256,6 +256,7 @@ struct gsm_network *msc_network_alloc(void *ctx,
 						  MSC_HLR_REMOTE_IP_DEFAULT);
 	net->gsup_server_port = MSC_HLR_REMOTE_PORT_DEFAULT;
 
+	net->mgw.mgw_pool = mgcp_client_pool_alloc(net);
 	mgcp_client_conf_init(&net->mgw.conf);
 	net->call_waiting = true;
 	net->lcls_permitted = false;
@@ -317,6 +318,11 @@ static int msc_vty_go_parent(struct vty *vty)
 		vty->node = CONFIG_NODE;
 		vty->index = NULL;
 		break;
+	case MGW_NODE:
+		OSMO_ASSERT(msc_network != NULL);
+		vty->node = GSMNET_NODE;
+		vty->index = msc_network;
+		break;
 	case SMPP_ESME_NODE:
 		vty->node = SMPP_NODE;
 		vty->index = NULL;
@@ -546,6 +552,41 @@ extern void *tall_gsms_ctx;
 extern void *tall_call_ctx;
 extern void *tall_trans_ctx;
 
+static int msc_mgw_setup(void)
+{
+	struct mgcp_client *mgcp_client_single;
+	unsigned int pool_members_initalized;
+
+	/* Initialize MGW pool. This initalizes and connects all MGCP clients that are currently configured in
+	 * the pool. Adding additional MGCP clients to the pool is possible but the user has to configure and
+	 * (re)connect them manually from the VTY. */
+	pool_members_initalized = mgcp_client_pool_connect(msc_network->mgw.mgw_pool);
+	if (pool_members_initalized) {
+		LOGP(DMSC, LOGL_NOTICE,
+		     "MGW pool with %u pool members configured, (ignoring MGW configuration in VTY node 'msc').\n",
+		     pool_members_initalized);
+		return 0;
+	}
+
+	/* Initialize and connect a single MGCP client. This MGCP client will appear as the one and only pool
+	 * member if there is no MGW pool configured. */
+	LOGP(DMSC, LOGL_NOTICE, "No MGW pool configured, using MGW configuration in VTY node 'msc'\n");
+	mgcp_client_single = mgcp_client_init(msc_network, &msc_network->mgw.conf);
+	if (!mgcp_client_single) {
+		LOGP(DMSC, LOGL_ERROR, "MGW (single) client initalization failed\n");
+		return -EINVAL;
+	}
+	if (mgcp_client_connect(mgcp_client_single)) {
+		LOGP(DMSC, LOGL_ERROR, "MGW (single) connect failed at (%s:%u)\n",
+		     msc_network->mgw.conf.remote_addr,
+		     msc_network->mgw.conf.remote_port);
+		return -EINVAL;
+	}
+	mgcp_client_pool_register_single(msc_network->mgw.mgw_pool, mgcp_client_single);
+
+	return 0;
+}
+
 int main(int argc, char **argv)
 {
 	int rc;
@@ -608,7 +649,7 @@ int main(int argc, char **argv)
 	}
 
 #ifdef BUILD_SMPP
-	if (smpp_openbsc_alloc_init(tall_msc_ctx) < 0)
+	if (smpp_msc_alloc_init(tall_msc_ctx) < 0)
 		return -1;
 #endif
 	sgs_iface_init(tall_msc_ctx, msc_network);
@@ -652,13 +693,12 @@ int main(int argc, char **argv)
 	 * following code until iu_init() is legacy. */
 
 #ifdef BUILD_SMPP
-	smpp_openbsc_start(msc_network);
+	smpp_msc_start(msc_network);
 #endif
 
 	/* start control interface after reading config for
 	 * ctrl_vty_get_bind_addr() */
-	msc_network->ctrl = ctrl_interface_setup_dynip(msc_network, ctrl_vty_get_bind_addr(),
-						       OSMO_CTRL_PORT_MSC, NULL);
+	msc_network->ctrl = ctrl_interface_setup(msc_network, OSMO_CTRL_PORT_MSC, NULL);
 	if (!msc_network->ctrl) {
 		fprintf(stderr, "Failed to initialize control interface. Exiting.\n");
 		return -1;
@@ -705,13 +745,8 @@ TODO: we probably want some of the _net_ ctrl commands from bsc_base_ctrl_cmds_i
 	if (sms_queue_start(msc_network) != 0)
 		return -1;
 
-	msc_network->mgw.client = mgcp_client_init(
-			msc_network, &msc_network->mgw.conf);
-
-	if (mgcp_client_connect(msc_network->mgw.client)) {
-		fprintf(stderr, "MGCPGW connect failed\n");
+	if (msc_mgw_setup() != 0)
 		return 7;
-	}
 
 	if (ss7_setup(tall_msc_ctx, &sccp_a, &sccp_iu)) {
 		fprintf(stderr, "Setting up SCCP client failed.\n");
